//> program_type = HLSL
//> entry = vs_main
//> target = vs_2_0
//> selector vertex_color = bool		// true to blend in vertex color
//> selector b_normal_map_texture = bool	// true if you want bump mapping
//> selector emissive_color = bool		// true if you want material emissive
//> selector b_reflection_map_texture = bool	// true if you want a reflective material
//> selector b_diffuse_texture = bool		// true if you want a diffuse texture
//> selector_max num_lights = 3
//> selector bool_shadow_casters = bool         // Engine Parameter 

float4x4 world_view_proj_matrix;		// Engine Parameter
float4x4 texture_matrix;			// Engine Parameter

struct VS_OUTPUT
{
	float4  Pos		: POSITION;
	float4  Color		: COLOR0;
	float4  ColorSpec	: COLOR1;
	float4  Tex0		: TEXCOORD0;
	float3	ObjPos		: TEXCOORD1;
	float3  Tangent		: TEXCOORD2;
	float3  Binormal	: TEXCOORD3;
	float3  Normal		: TEXCOORD4;
	float3  SxT		: TEXCOORD5;
#if bool_shadow_casters
        float4  ScreenTexPos    : TEXCOORD6;
#endif
};

struct COLOR_PAIR
{
	float4 Color		: COLOR0;
	float4 ColorSpec	: COLOR1;
};

// Transform Matrices

// Material Colors
float4	 cDiffuseMaterial;			// Diffuse Material Color
float4	 cSpecularMaterial;			// Specular Material Color
float4	 cEmissiveMaterial;			// Emissive Material Color

VS_OUTPUT vs_main( float4 vPosition : POSITION0,
			float3 vNormal : NORMAL0,
			float2 vTexCoord : TEXCOORD0,
			float4 vColor : COLOR0,
			float3 vTangent : TANGENT,
			float3 vBinormal : BINORMAL,
			float3 vSxT : TEXCOORD1 )
{
	VS_OUTPUT Out = (VS_OUTPUT) 0;
	
	// Transform Position
	Out.Pos = mul( world_view_proj_matrix, vPosition );
	Out.ObjPos = vPosition.xyz;
	
 	// If they specified an emissive material, just use that color.
	#if emissive_color
		Out.Color = cEmissiveMaterial;
		Out.ColorSpec = 0;
	#endif
	
	#if emissive_color == 0
		Out.ColorSpec = cSpecularMaterial;
			
		#if vertex_color
			# if b_diffuse_texture
				Out.Color = vColor;
			#endif

			#if b_diffuse_texture == 0
				Out.Color = vColor * cDiffuseMaterial;
			#endif
		#endif
			
		#if vertex_color == 0
			Out.Color = cDiffuseMaterial; 
		#endif
	#endif
	
	// Just pass thru the texture coords
	Out.Tex0.x = vTexCoord.x;
	Out.Tex0.y = vTexCoord.y;
	Out.Tex0.zw = 0.0f;

	#if bool_shadow_casters
		Out.ScreenTexPos = mul( texture_matrix, Out.Pos );
	#endif
	
	#if b_normal_map_texture
		Out.Tangent = vTangent;
		Out.Binormal = vBinormal;
		Out.SxT = vSxT;
	#endif
	
	Out.Normal = vNormal;
	      
 	return Out;
}
